@@ -10,7 +10,12 @@ import serial.tools.list_ports
 import voluptuous as vol
 
 from ledfx.config import save_config
-from ledfx.events import DeviceCreatedEvent, DeviceUpdateEvent, Event
+from ledfx.events import (
+    DeviceCreatedEvent,
+    DevicesUpdatedEvent,
+    DeviceUpdateEvent,
+    Event,
+)
 from ledfx.utils import (
     AVAILABLE_FPS,
     WLED,
@@ -181,6 +186,11 @@ class Device(BaseRegistry):
         self._active = False
         # self.flush(np.zeros((self.pixel_count, 3)))
 
+    def set_offline(self):
+        self.deactivate()
+        self._online = False
+        self._ledfx.events.fire_event(DevicesUpdatedEvent(self.id))
+
     @abstractmethod
     def flush(self, data):
         """
@@ -241,9 +251,7 @@ class Device(BaseRegistry):
     @property
     def online(self):
         """
-        list of id of the virtuals active on this device.
-        it's a list bc there can be more than one virtual streaming
-        to a device.
+        bool indicator of online status
         """
         return self._online
 
@@ -557,13 +565,13 @@ class SerialDevice(Device):
             self.serial = serial.Serial(self.com_port, self.baudrate)
             if self.serial.isOpen:
                 super().activate()
+                self._online = True
 
         except serial.SerialException:
-            _LOGGER.critical(
+            _LOGGER.warning(
                 "Serial Error: Please ensure your device is connected, functioning and the correct COM port is selected."
             )
-            # Todo: Trigger the UI to refresh after the clear effect call. Currently it still shows as active.
-            self.deactivate()
+            self.set_offline()
 
     def deactivate(self):
         super().deactivate()
@@ -633,7 +641,7 @@ class Devices(RegistryLoader):
                     self._ledfx.loop, self._ledfx.thread_executor, device_ip
                 )
             except ValueError:
-                _LOGGER.error(f"Discarding device {device_ip}")
+                _LOGGER.warning(f"Discarding device {device_ip}")
                 return
 
             for existing_device in self._ledfx.devices.values():
