@@ -112,12 +112,12 @@ class LaunchpadDevice(MidiDevice):
                     default=False,
                 ): bool,
                 vol.Optional(
-                    "Alpha",
+                    "alpha_options",
                     description="Dark and dangerous features of the damned",
                     default=False,
                 ): bool,
                 vol.Optional(
-                    "Diagnostic",
+                    "diag",
                     description="enable timing diagnostics in logger",
                     default=False,
                 ): bool,
@@ -127,27 +127,56 @@ class LaunchpadDevice(MidiDevice):
     def __init__(self, ledfx, config):
         super().__init__(ledfx, config)
         self.lp = None
+        self.lp_name = "unknown"
         _LOGGER.info("Launchpad device created")
 
     def flush(self, data):
-        self.lp.flush(data, self._config["Alpha"], self._config["Diagnostic"])
+        success = self.lp.flush(
+            data, self._config["alpha_options"], self._config["diag"]
+        )
+        if not success:
+            _LOGGER.warning(
+                f"Error in Launchpad {self.lp_name} flush, setting offline"
+            )
+            self.set_offline()
 
     def activate(self):
         self.set_class()
-        super().activate()
+        if self.lp is not None:
+            if self.lp.supported:
+                self._online = True
+                super().activate()
+            else:
+                _LOGGER.warning(
+                    "Launchpad variant not supported or not connected"
+                )
+                self.set_offline()
+        else:
+            self.set_offline()
 
     def set_class(self):
         self.lp = launchpad.Launchpad()
-        self.validate_launchpad()
-        _LOGGER.info(f"Launchpad device class: {self.lp.__class__.__name__}")
+        self.lp_name = self.validate_launchpad()
+
+        if self.lp_name is None:
+            _LOGGER.warning(
+                f"Launchpad validation failed class: {self.lp.__class__.__name__}"
+            )
+            self.lp = None
+        else:
+            _LOGGER.info(
+                f"Launchpad {self.lp_name} device class: {self.lp.__class__.__name__}"
+            )
 
     def deactivate(self):
+        _LOGGER.info("Deactivating Launchpad")
         if self.lp is not None:
             self.lp.flush(
                 zeros((self.pixel_count, 3)),
-                self._config["Alpha"],
-                self._config["Diagnostic"],
+                self._config["alpha_options"],
+                self._config["diag"],
             )
+            _LOGGER.info("Closing Launchpad")
             self.lp.Close()
             self.lp = None
         super().deactivate()
