@@ -1,4 +1,3 @@
-import colorsys
 import logging
 import threading
 
@@ -7,8 +6,9 @@ from functools import lru_cache
 
 import numpy as np
 import voluptuous as vol
+from numpy.typing import NDArray
 
-from ledfx.color import parse_color, validate_color
+from ledfx.color import hsv_to_rgb, parse_color, validate_color
 from ledfx.utils import BaseRegistry, RegistryLoader
 
 _LOGGER = logging.getLogger(__name__)
@@ -47,7 +47,7 @@ class DummyEffect:
         pass
 
 
-def mix_colors(color_1, color_2, ratio):
+def mix_colors(color_1: tuple, color_2: tuple, ratio: float) -> tuple:
     """
     Mixes two colors based on a given ratio.
 
@@ -73,7 +73,9 @@ def mix_colors(color_1, color_2, ratio):
         )
 
 
-def fill_rainbow(pixels, initial_hue, delta_hue):
+def fill_rainbow(
+    pixels: NDArray, initial_hue: float, delta_hue: float
+) -> NDArray:
     """
     Fills the given pixels with a rainbow effect.
 
@@ -87,17 +89,21 @@ def fill_rainbow(pixels, initial_hue, delta_hue):
     """
     sat = 0.95
     val = 1.0
+
+    # Create an array of hue values starting from 'initial_hue' and increasing
+    # by 'delta_hue' for each pixel. The array length is initially set to be longer
+    # than the number of pixels.
     hues = np.arange(
         initial_hue, initial_hue + len(pixels) * delta_hue, delta_hue
     )
-    hues = hues[: len(pixels)]  # Ensure hues has the same length as pixels
-    pixels = (
-        np.array([colorsys.hsv_to_rgb(hue, sat, val) for hue in hues]) * 255
-    )  # the array is 0-1, we need 0-255
-    return pixels
+
+    # ensure each pixel has a corresponding hue value.
+    hues = hues[: len(pixels)]
+
+    return hsv_to_rgb(hues, sat, val)
 
 
-def blur_pixels(pixels, sigma):
+def blur_pixels(pixels: NDArray, sigma: float) -> NDArray:
     """
     Applies a blur effect to the given pixels.
 
@@ -116,7 +122,7 @@ def blur_pixels(pixels, sigma):
 
 
 @lru_cache(maxsize=1024)
-def _gaussian_kernel1d(sigma, order, array_len):
+def _gaussian_kernel1d(sigma: float, order: int, array_len: int) -> NDArray:
     """
     Produces a 1D Gaussian or Gaussian-derivative filter kernel as a numpy array.
 
@@ -161,7 +167,7 @@ def _gaussian_kernel1d(sigma, order, array_len):
     return phi_x
 
 
-def fast_blur_pixels(pixels, sigma):
+def fast_blur_pixels(pixels: NDArray, sigma: float) -> NDArray:
     """
     Applies a fast blur effect to the given pixels using a Gaussian kernel.
 
@@ -184,7 +190,7 @@ def fast_blur_pixels(pixels, sigma):
     return pixels
 
 
-def fast_blur_array(array, sigma):
+def fast_blur_array(array: NDArray, sigma: float) -> NDArray:
     """
     Apply fast Gaussian blur to a 1-dimensional array.
 
@@ -321,28 +327,25 @@ class Effect(BaseRegistry):
             self._deactivate()
 
     def activate(self, virtual):
-        self.lock.acquire()
         """Attaches an output channel to the effect"""
-        self._virtual = virtual
-        self.pixels = np.zeros((virtual.pixel_count, 3))
-        # Iterate all the base classes and check to see if the base
-        # class has an on_activate method. If so, call it
-        valid_classes = list(type(self).__bases__)
-        valid_classes.append(type(self))
-        for base in valid_classes:
-            if hasattr(base, "on_activate"):
-                base.on_activate(self, virtual.pixel_count)
-
-        self._active = True
-        self.lock.release()
-        _LOGGER.info(f"Effect {self.NAME} activated.")
+        with self.lock:
+            self._virtual = virtual
+            self.pixels = np.zeros((virtual.effective_pixel_count, 3))
+            # Iterate all the base classes and check to see if the base
+            # class has an on_activate method. If so, call it
+            valid_classes = list(type(self).__bases__)
+            valid_classes.append(type(self))
+            for base in valid_classes:
+                if hasattr(base, "on_activate"):
+                    base.on_activate(self, virtual.effective_pixel_count)
+            self._active = True
+            _LOGGER.info(f"Effect {self.NAME} activated.")
 
     def _deactivate(self):
         # we need this wrapper to ensure the full chain of
         # deactivation is protected
-        self.lock.acquire()
-        self.deactivate()
-        self.lock.release()
+        with self.lock:
+            self.deactivate()
 
     def deactivate(self):
         """Detaches an output channel from the effect"""
@@ -367,43 +370,43 @@ class Effect(BaseRegistry):
         return combined_schema
 
     def update_config(self, config):
-        self.lock.acquire()
-        try:
-            validated_config = type(self).schema()(config)
-        except vol.Invalid as err:
-            _LOGGER.warning(f"Error updating effect {self.NAME} config: {err}")
-            self.lock.release()
-            return
+        with self.lock:
+            try:
+                validated_config = type(self).schema()(config)
+            except vol.Invalid as err:
+                _LOGGER.warning(
+                    f"Error updating effect {self.NAME} config: {err}"
+                )
+                return
 
-        prior_config = self._config
+            prior_config = self._config
 
-        if self._config != {}:
-            self._config = {**prior_config, **config}
-        else:
-            self._config = validated_config
+            if self._config != {}:
+                self._config = {**prior_config, **config}
+            else:
+                self._config = validated_config
 
-        self._bg_color = (
-            np.array(parse_color(self._config["background_color"]))
-            * self._config["background_brightness"]
-        )
+            self._bg_color = (
+                np.array(parse_color(self._config["background_color"]))
+                * self._config["background_brightness"]
+            )
 
-        def inherited(cls, method):
-            if hasattr(cls, method) and hasattr(super(cls, cls), method):
-                return cls.foo == super(cls).foo
-            return False
+            def inherited(cls, method):
+                if hasattr(cls, method) and hasattr(super(cls, cls), method):
+                    return cls.foo == super(cls).foo
+                return False
 
-        # Iterate all the base classes and check to see if there is a custom
-        # implementation of config updates. If to notify the base class.
-        valid_classes = list(type(self).__bases__)
-        valid_classes.append(type(self))
-        for base in valid_classes:
-            if base.config_updated != super(base, base).config_updated:
-                base.config_updated(self, self._config)
-        self.lock.release()
+            # Iterate all the base classes and check to see if there is a custom
+            # implementation of config updates. If to notify the base class.
+            valid_classes = list(type(self).__bases__)
+            valid_classes.append(type(self))
+            for base in valid_classes:
+                if base.config_updated != super(base, base).config_updated:
+                    base.config_updated(self, self._config)
 
-        _LOGGER.debug(
-            f"Effect {self.NAME} config updated to {validated_config}."
-        )
+            _LOGGER.debug(
+                f"Effect {self.NAME} config updated to {validated_config}."
+            )
 
     def config_updated(self, config):
         """
@@ -415,11 +418,10 @@ class Effect(BaseRegistry):
         pass
 
     def _render(self):
-        self.lock.acquire()
-        # its possible we were waiting on the effect being deactivated
-        if self._active:
-            self.render()
-        self.lock.release()
+        with self.lock:
+            # its possible we were waiting on the effect being deactivated
+            if self._active:
+                self.render()
 
     def render(self):
         """
@@ -431,55 +433,62 @@ class Effect(BaseRegistry):
         pass
 
     def get_pixels(self):
-        self.lock.acquire()
-        pixels = None
-        if hasattr(self, "pixels"):
-            if self.pixels is not None:
-                pixels = np.copy(self.pixels)
-                # Grab the config and store it here for use in the function - we use it a lot
-                config = self._config
+        """
+        Get the current pixels for the effect and apply flip, mirror, blur, brightness and background color transformations
 
-                # Apply some of the base output filters if necessary
-                if config["flip"]:
-                    pixels = np.flipud(pixels)
-                if config["mirror"]:
-                    pixels = np.concatenate(
-                        (pixels[-1 + len(pixels) % -2 :: -2], pixels[::2])
-                    )
-                if config["background_color"]:
-                    pixels += self._bg_color
-                if config["brightness"] is not None:
-                    np.multiply(
-                        pixels,
-                        config["brightness"],
-                        out=pixels,
-                        casting="unsafe",
-                    )
+        Returns:
+            numpy.ndarray: The modified pixel array.
+        """
+        with self.lock:
+            pixels = None
+            if hasattr(self, "pixels"):
+                if self.pixels is not None:
+                    pixels = np.copy(self.pixels)
+                    # Grab the config and store it here for use in the function - we use it a lot
+                    config = self._config
 
-                # If the configured blur is greater than 0 and pixel_count > 3, apply blur
-                # The matrix math requires > 3 pixels to work properly
-                # And blurring with a less than 3 pixels seems... redundant
-                # TODO: Handle RGBW properly
-                if config["blur"] != 0.0 and self.pixel_count > 3:
-                    kernel = _gaussian_kernel1d(config["blur"], 0, len(pixels))
+                    # Apply some of the base output filters if necessary
+                    if config["flip"]:
+                        pixels = np.flipud(pixels)
+                    if config["mirror"]:
+                        pixels = np.concatenate(
+                            (pixels[-1 + len(pixels) % -2 :: -2], pixels[::2])
+                        )
+                    if config["background_color"]:
+                        pixels += self._bg_color
+                    if config["brightness"] is not None:
+                        np.multiply(
+                            pixels,
+                            config["brightness"],
+                            out=pixels,
+                            casting="unsafe",
+                        )
 
-                    # Blur the R,G,B portions of the pixel array
-                    # Lots of attempts at vectorisation/performance improvements here
-                    # This appears to be optimal from a readability/performance point of view
-                    # TODO: If we ever move to RGBW pixel arrays, uncomment the last line to operate on the W portion
+                    # If the configured blur is greater than 0 and pixel_count > 3, apply blur
+                    # The matrix math requires > 3 pixels to work properly
+                    # And blurring with a less than 3 pixels seems... redundant
+                    # TODO: Handle RGBW properly
+                    if config["blur"] != 0.0 and self.pixel_count > 3:
+                        kernel = _gaussian_kernel1d(
+                            config["blur"], 0, len(pixels)
+                        )
 
-                    pixels[:, 0] = np.convolve(
-                        pixels[:, 0], kernel, mode="same"
-                    )  # R
-                    pixels[:, 1] = np.convolve(
-                        pixels[:, 1], kernel, mode="same"
-                    )  # G
-                    pixels[:, 2] = np.convolve(
-                        pixels[:, 2], kernel, mode="same"
-                    )  # B
-                    # pixels[:, 3] = np.convolve(pixels[:, 3], kernel, mode="same") # W
-        self.lock.release()
-        return pixels
+                        # Blur the R,G,B portions of the pixel array
+                        # Lots of attempts at vectorisation/performance improvements here
+                        # This appears to be optimal from a readability/performance point of view
+                        # TODO: If we ever move to RGBW pixel arrays, uncomment the last line to operate on the W portion
+
+                        pixels[:, 0] = np.convolve(
+                            pixels[:, 0], kernel, mode="same"
+                        )  # R
+                        pixels[:, 1] = np.convolve(
+                            pixels[:, 1], kernel, mode="same"
+                        )  # G
+                        pixels[:, 2] = np.convolve(
+                            pixels[:, 2], kernel, mode="same"
+                        )  # B
+                        # pixels[:, 3] = np.convolve(pixels[:, 3], kernel, mode="same") # W
+                return pixels
 
     @property
     def is_active(self):
