@@ -53,6 +53,13 @@ class AudioInputSource:
         """
         return tuple(AudioInputSource.input_devices().keys())
 
+    @staticmethod
+    def audio_input_device_exists():
+        """
+        Returns True if there are valid input devices
+        """
+        return len(AudioInputSource.valid_device_indexes()) > 0
+
     @staticmethod
     def default_device_index():
         """
@@ -68,12 +75,34 @@ class AudioInputSource:
             f"{device_list[default_output_device]['name']} [Loopback]"
         )
         # We need to run over the device list looking for the target device
+        # NOTE: Some sound drivers truncate the device name, so we may not find a match
         for device_index, device in enumerate(device_list):
             if device["name"] == target_device:
                 # Return the loopback device index
+                _LOGGER.debug(
+                    f"Default audio loopback device found: {device['name']}"
+                )
                 return device_index
-        # No Loopback device matching output found - return the default input device index
-        return sd.default.device["input"]
+        # If we don't match a Loopback device matching output found - return the default input device index
+        default_input_device_idx = sd.default.device["input"]
+        # The default input device index is not always valid (i.e no default input devices)
+        if default_input_device_idx in AudioInputSource.valid_device_indexes():
+            _LOGGER.debug(
+                "No default audio loopback device found. Using default input device."
+            )
+            return default_input_device_idx
+        else:
+            # Return the first valid input device index if we can't find a valid default input device
+            if len(AudioInputSource.valid_device_indexes()) > 0:
+                _LOGGER.debug(
+                    "No valid default audio input device found. Using first valid input device."
+                )
+                return next(iter(AudioInputSource.valid_device_indexes()))
+            else:
+                _LOGGER.warning(
+                    "No valid audio input devices found. Unable to use audio reactive effects."
+                )
+                return None
 
     @staticmethod
     def query_hostapis():
@@ -110,7 +139,8 @@ class AudioInputSource:
         default_device_index = AudioInputSource.default_device_index()
         valid_device_indexes = AudioInputSource.valid_device_indexes()
         input_devices = AudioInputSource.input_devices()
-
+        melbanks = Melbanks.CONFIG_SCHEMA
+        audio_analysis = AudioAnalysisSource.CONFIG_SCHEMA
         return vol.Schema(
             {
                 vol.Optional("sample_rate", default=60): int,
@@ -133,6 +163,7 @@ class AudioInputSource:
 
     def __init__(self, ledfx, config):
         self._ledfx = ledfx
+        self.lock = threading.Lock()
         self.update_config(config)
 
         def shutdown_event(e):
@@ -164,6 +195,7 @@ class AudioInputSource:
                     self.input_devices()[self._config["audio_device"]]
                 )
             )
+        self._ledfx.config["audio"] = self._config
 
     def activate(self):
         if self._audio is None:
@@ -178,6 +210,15 @@ class AudioInputSource:
         input_devices = self.query_devices()
         hostapis = self.query_hostapis()
         default_device = self.default_device_index()
+        if default_device is None:
+            # There are no valid audio input devices, so we can't activate the audio source.
+            # We should never get here, as we check for devices on start-up.
+            # This likely just captures if a device is removed after start-up.
+            _LOGGER.warning(
+                "Audio input device not found. Unable to activate audio source. Deactivating."
+            )
+            self.deactivate()
+            return
         valid_device_indexes = self.valid_device_indexes()
         device_idx = self._config["audio_device"]
 
@@ -193,50 +234,26 @@ class AudioInputSource:
             )
             device_idx = default_device
 
-        # hostapis = self._audio.query_hostapis()
-        # devices = self._audio.query_devices()
-        # default_api = self._audio.default.hostapi
-
-        # Show device and api info in logger
-        # _LOGGER.debug("Audio Input Devices:")
-        # for api_idx, api in enumerate(hostapis):
-        #     _LOGGER.debug(
-        #         "Host API: {} {}".format(
-        #             api["name"],
-        #             "[DEFAULT]" if api_idx == default_api else "",
-        #         )
-        #     )
-        #     for idx in api["devices"]:
-        #         device = devices[idx]
-        #         if device["max_input_channels"] > 0:
-        #             _LOGGER.debug(
-        #                 "    [{}] {} {} {}".format(
-        #                     idx,
-        #                     device["name"],
-        #                     "[DEFAULT]" if idx == default_device else "",
-        #                     "[SELECTED]" if idx == device_idx else "",
-        #                 )
-        #             )
-
-        # old, do not use
-        # self.pre_emphasis.set_biquad(1., -self._config['pre_emphasis'], 0, 0, 0)
-
-        # USE THESE FOR SCOTT_MEL OR OTHERS
-        # self.pre_emphasis.set_biquad(1.3662, -1.9256, 0.5621, -1.9256, 0.9283)
-
-        # USE THESE FOR MATT_MEl
-        # weaker bass, good for vocals, highs
-        # self.pre_emphasis.set_biquad(0.87492, -1.74984, 0.87492, -1.74799, 0.75169)
-        # bass heavier overall more balanced
-        # self.pre_emphasis.set_biquad(
-        #     0.85870, -1.71740, 0.85870, -1.71605, 0.71874
-        # )
-
         # Setup a pre-emphasis filter to balance the input volume of lows to highs
         self.pre_emphasis = aubio.digital_filter(3)
-        self.pre_emphasis.set_biquad(0.8268, -1.6536, 0.8268, -1.6536, 0.6536)
+        # depending on the coeffs type, we need to use different pre_emphasis values to make em work better. allegedly.
+        selected_coeff = self._ledfx.config["melbanks"]["coeffs_type"]
+        if selected_coeff == "matt_mel":
+            _LOGGER.debug("Using matt_mel settings for pre-emphasis.")
+            self.pre_emphasis.set_biquad(
+                0.8268, -1.6536, 0.8268, -1.6536, 0.6536
+            )
+        elif selected_coeff == "scott_mel":
+            _LOGGER.debug("Using scott_mel settings for pre-emphasis.")
+            self.pre_emphasis.set_biquad(
+                1.3662, -1.9256, 0.5621, -1.9256, 0.9283
+            )
+        else:
+            _LOGGER.debug("Using generic settings for pre-emphasis")
+            self.pre_emphasis.set_biquad(
+                0.85870, -1.71740, 0.85870, -1.71605, 0.71874
+            )
 
-        # self.pre_emphasis = None,
         freq_domain_length = (self._config["fft_size"] // 2) + 1
 
         self._raw_audio_sample = np.zeros(
@@ -273,10 +290,10 @@ class AudioInputSource:
                     ch = 2
 
             if hostapis[device["hostapi"]]["name"] == "WEB AUDIO":
-                ledfx.api.websocket.ACTIVE_AUDIO_STREAM = (
-                    self._stream
-                ) = WebAudioStream(
-                    device["client"], self._audio_sample_callback
+                ledfx.api.websocket.ACTIVE_AUDIO_STREAM = self._stream = (
+                    WebAudioStream(
+                        device["client"], self._audio_sample_callback
+                    )
                 )
             else:
                 self._stream = self._audio.InputStream(
@@ -313,11 +330,12 @@ class AudioInputSource:
             open_audio_stream(default_device)
 
     def deactivate(self):
-        if self._stream:
-            self._stream.stop()
-            self._stream.close()
-            self._stream = None
-        self._is_activated = False
+        with self.lock:
+            if self._stream:
+                self._stream.stop()
+                self._stream.close()
+                self._stream = None
+            self._is_activated = False
         _LOGGER.info("Audio source closed.")
 
     def subscribe(self, callback):
@@ -372,14 +390,14 @@ class AudioInputSource:
             processed_audio_sample = self.resampler.process(
                 raw_sample,
                 # MIC_RATE / self._stream.samplerate
-                out_sample_len / in_sample_len
+                out_sample_len / in_sample_len,
                 # end_of_input=True
             )
         else:
             processed_audio_sample = raw_sample
 
         if len(processed_audio_sample) != out_sample_len:
-            _LOGGER.warning(
+            _LOGGER.debug(
                 f"Discarded malformed audio frame - {len(processed_audio_sample)} samples, expected {out_sample_len}"
             )
             return
@@ -462,12 +480,47 @@ class AudioInputSource:
 
 
 class AudioAnalysisSource(AudioInputSource):
+    # https://aubio.org/doc/latest/pitch_8h.html
+    PITCH_METHODS = [
+        "yinfft",
+        "yin",
+        "yinfast",
+        # mcomb and fcomb appears to just explode something deeep in the aubio code, no logs, no errors, it just dies.
+        # "mcomb",
+        # "fcomb",
+        "schmitt",
+        "specacf",
+    ]
+    # https://aubio.org/doc/latest/specdesc_8h.html
+    ONSET_METHODS = [
+        "energy",
+        "hfc",
+        "complex",
+        "phase",
+        "wphase",
+        "specdiff",
+        "kl",
+        "mkl",
+        "specflux",
+    ]
     CONFIG_SCHEMA = vol.Schema(
         {
-            vol.Optional("pitch_method", default="default"): str,
+            vol.Optional(
+                "pitch_method",
+                default="yinfft",
+                description="Method to detect pitch",
+            ): vol.In(PITCH_METHODS),
             vol.Optional("tempo_method", default="default"): str,
-            vol.Optional("onset_method", default="specflux"): str,
-            vol.Optional("pitch_tolerance", default=0.8): float,
+            vol.Optional(
+                "onset_method",
+                default="hfc",
+                description="Method used to detect onsets",
+            ): vol.In(ONSET_METHODS),
+            vol.Optional(
+                "pitch_tolerance",
+                default=0.8,
+                description="Pitch detection tolerance",
+            ): vol.All(vol.Coerce(float), vol.Range(min=0.0, max=2)),
         },
         extra=vol.ALLOW_EXTRA,
     )
@@ -532,7 +585,7 @@ class AudioAnalysisSource(AudioInputSource):
         self.freq_mel_indexes = []
 
         for freq in self.freq_max_mels:
-            assert self.melbanks._config["max_frequencies"][2] >= freq
+            assert self.melbanks.melbanks_config["max_frequencies"][2] >= freq
 
             self.freq_mel_indexes.append(
                 next(
@@ -673,9 +726,11 @@ class AudioAnalysisSource(AudioInputSource):
 
     def get_freq_power(self, i, filtered=True):
         if filtered:
-            return self.freq_power_filter.value[i]
+            value = self.freq_power_filter.value[i]
         else:
-            return self.freq_power_raw[i]
+            value = self.freq_power_raw[i]
+
+        return value if not np.isnan(value) else 0.0
 
     def beat_power(self, filtered=True):
         """
@@ -780,6 +835,11 @@ class AudioReactiveEffect(Effect):
         "High": "high_power",
     }
 
+    def __init__(self, ledfx, config):
+        super().__init__(ledfx, config)
+        # protect against possible deactivate race condition
+        self.audio = None
+
     def activate(self, channel):
         _LOGGER.info("Activating AudioReactiveEffect.")
         super().activate(channel)
@@ -809,10 +869,9 @@ class AudioReactiveEffect(Effect):
 
     def _audio_data_updated(self):
         self.melbank.cache_clear()
-        self.lock.acquire()
-        if self.is_active:
-            self.audio_data_updated(self.audio)
-        self.lock.release()
+        with self.lock:
+            if self.is_active:
+                self.audio_data_updated(self.audio)
 
     def audio_data_updated(self, data):
         """
@@ -845,11 +904,11 @@ class AudioReactiveEffect(Effect):
             (
                 i
                 for i, x in enumerate(
-                    self.audio.melbanks._config["max_frequencies"]
+                    self.audio.melbanks.melbanks_config["max_frequencies"]
                 )
                 if x >= self._virtual.frequency_range.max
             ),
-            len(self.audio.melbanks._config["max_frequencies"]),
+            len(self.audio.melbanks.melbanks_config["max_frequencies"]),
         )
 
     @cached_property
@@ -893,6 +952,18 @@ class AudioReactiveEffect(Effect):
         new = np.linspace(0, 1, size)
         return (new, old)
 
+    def melbank_no_nan(self, melbank):
+        # Check for NaN values in the melbank array, replace with 0 in place
+        # Difficult to determine why this happens, but it seems to be related to
+        # the audio input device.
+        # TODO: Investigate why NaNs are present in the melbank array for some people/devices
+        if np.isnan(melbank).any():
+            _LOGGER.warning(
+                "NaN values detected in the melbank array and replaced with 0."
+            )
+            # Replace NaN values with 0
+            np.nan_to_num(melbank, copy=False)
+
     @lru_cache(maxsize=None)
     def melbank(self, filtered=False, size=0):
         """
@@ -911,6 +982,9 @@ class AudioReactiveEffect(Effect):
             melbank = self.audio.melbanks.melbanks[self._selected_melbank][
                 self._melbank_min_idx : self._melbank_max_idx
             ]
+
+        self.melbank_no_nan(melbank)
+
         if size and (self._input_mel_length != size):
             return np.interp(*self._melbank_interp_linspaces(size), melbank)
         else:
@@ -925,14 +999,4 @@ class AudioReactiveEffect(Effect):
         mel_length = len(melbank)
         splits = tuple(map(lambda i: int(i * mel_length), [0.2, 0.5]))
 
-        # Check for NaN values in the melbank array
-        # Difficult to determine why this happens, but it seems to be related to
-        # the audio input device. If NaNs are present, replace them with 0
-        # TODO: Investigate why NaNs are present in the melbank array for some people/devices
-        if np.isnan(melbank).any():
-            _LOGGER.warning(
-                "NaN values detected in the melbank array and replaced with 0."
-            )
-            # Replace NaN values with 0
-            melbank = np.nan_to_num(melbank)
         return np.split(melbank, splits)
