@@ -14,24 +14,25 @@ _LOGGER = logging.getLogger(__name__)
 @Effect.no_registration
 class Twod(AudioReactiveEffect):
     EFFECT_START_TIME = timeit.default_timer()
-    HIDDEN_KEYS = ["background_brightness", "mirror", "flip", "blur"]
+    # hiding dump by default, a dev can turn it on explicitily via removal
+    HIDDEN_KEYS = ["background_brightness", "mirror", "flip", "blur", "dump"]
     ADVANCED_KEYS = [
         "dump",
         "diag",
         "test",
-        "flip horizontal",
-        "flip vertical",
+        "flip_horizontal",
+        "flip_vertical",
     ]
 
     CONFIG_SCHEMA = vol.Schema(
         {
             vol.Optional(
-                "flip horizontal",
+                "flip_horizontal",
                 description="flip the image horizontally",
                 default=False,
             ): bool,
             vol.Optional(
-                "flip vertical",
+                "flip_vertical",
                 description="flip the image vertically",
                 default=False,
             ): bool,
@@ -66,6 +67,7 @@ class Twod(AudioReactiveEffect):
     def __init__(self, ledfx, config):
         super().__init__(ledfx, config)
         self.lasttime = 0
+        self.passed = 0
         self.frame = 0
         self.fps = 0
         self.last = 0
@@ -76,7 +78,7 @@ class Twod(AudioReactiveEffect):
         self.current_pixel = 0
         self.last_cycle_time = 20
         self.bar = 0
-        self.t_height = self._virtual.config["rows"]
+        self.t_height = max(1, self._virtual.config["rows"])
         self.t_width = self.pixel_count // self.t_height
         # initialise here so inherited can assume it exists
         self.current_time = timeit.default_timer()
@@ -91,8 +93,8 @@ class Twod(AudioReactiveEffect):
         # as well as a small performance boost
         # we need to accout for swapping vertical and horizotal for 90 / 270
 
-        self.flip = self._config["flip vertical"]
-        self.mirror = self._config["flip horizontal"]
+        self.flip = self._config["flip_vertical"]
+        self.mirror = self._config["flip_horizontal"]
 
         self.rotate = self._config["rotate"]
         self.rotate_t = 0
@@ -122,7 +124,7 @@ class Twod(AudioReactiveEffect):
         # also triggered by config change in parent virtual
         # presently only on row change
 
-        self.t_height = self._virtual.config["rows"]
+        self.t_height = max(1, self._virtual.config["rows"])
         self.t_width = self.pixel_count // self.t_height
 
         if self.rotate == 1 or self.rotate == 3:
@@ -230,7 +232,9 @@ class Twod(AudioReactiveEffect):
         pass
 
     def render(self):
+        was = self.current_time
         self.current_time = timeit.default_timer()
+        self.passed = self.current_time - was
         if self.init:
             self.do_once()
         # Update the time every frame
