@@ -9,6 +9,7 @@ _LOGGER = logging.getLogger(__name__)
 class Event:
     """Base for events"""
 
+    BASE_CONFIG_UPDATE = "base_config_update"
     LEDFX_SHUTDOWN = "shutdown"
     DEVICE_CREATED = "device_created"
     DEVICE_UPDATE = "device_update"
@@ -52,7 +53,9 @@ class DeviceCreatedEvent(Event):
 
 
 class DevicesUpdatedEvent(Event):
-    """Weird event emitted when OpenRGB device fails to connect"""
+    """
+    Event emitted when a device changes status due to something outside the users control - this is used to update the frontend
+    """
 
     def __init__(self, device_id: str):
         super().__init__(Event.DEVICES_UPDATED)
@@ -65,7 +68,6 @@ class VirtualUpdateEvent(Event):
     def __init__(self, virtual_id: str, pixels: np.ndarray):
         super().__init__(Event.VIRTUAL_UPDATE)
         self.virtual_id = virtual_id
-        # self.pixels = pixels.astype(np.uint8).T.tolist()
         self.pixels = pixels
 
 
@@ -166,6 +168,16 @@ class VirtualConfigUpdateEvent(Event):
         self.config = config
 
 
+class BaseConfigUpdateEvent(Event):
+    """
+    Event emitted when an item in the base configuration is updated.
+    """
+
+    def __init__(self, config):
+        super().__init__(Event.BASE_CONFIG_UPDATE)
+        self.config = config
+
+
 class LedFxShutdownEvent(Event):
     """Event emitted when LedFx is shutting down"""
 
