@@ -48,6 +48,11 @@ class Virtual:
                 description="Span: Effect spans all segments. Copy: Effect copied on each segment",
                 default="span",
             ): vol.In(["span", "copy"]),
+            vol.Required(
+                "grouping",
+                description="Number of physical pixels to combine into larger virtual pixel groups",
+                default=1,
+            ): vol.All(int, vol.Range(min=0)),
             vol.Optional(
                 "icon_name",
                 description="Icon for the device*",
@@ -153,6 +158,7 @@ class Virtual:
         self._hl_step = 1
         self._os_active = False
         self.lock = threading.Lock()
+        self.clear_handle = None
 
         self.frequency_range = FrequencyRange(
             self._config["frequency_min"], self._config["frequency_max"]
@@ -205,13 +211,27 @@ class Virtual:
             valid = False
 
         if (
-            (start_pixel < 0)
-            or (end_pixel < 0)
-            or (start_pixel > end_pixel)
-            or (end_pixel >= device.pixel_count)
+            start_pixel < 0
+            or end_pixel < 0
+            or start_pixel > end_pixel
+            or start_pixel >= device.pixel_count
+            or end_pixel >= device.pixel_count
         ):
-            msg = f"Invalid segment pixels: ({start_pixel}, {end_pixel}). Device '{self.name}' valid pixels between (0, {self.pixel_count - 1})"
-            valid = False
+            _LOGGER.warning(
+                f"Invalid segment pixels in Virtual '{self.name}': segment('{device.name}' ({start_pixel}, {end_pixel})) valid pixels between (0, {device.pixel_count - 1})"
+            )
+            if start_pixel < 0:
+                start_pixel = 0
+            if end_pixel < 0:
+                end_pixel = 0
+            if start_pixel > end_pixel:
+                start_pixel = end_pixel
+            if start_pixel >= device.pixel_count:
+                start_pixel = device.pixel_count - 1
+            if end_pixel >= device.pixel_count:
+                end_pixel = device.pixel_count - 1
+            segment = [device_id, start_pixel, end_pixel, invert]
+            _LOGGER.warning(f"Fixed to {segment}")
 
         if not valid:
             _LOGGER.error(msg)
@@ -226,55 +246,84 @@ class Virtual:
             "refresh_rate",
             "_devices",
             "_segments_by_device",
+            "effective_pixel_count",
+            "group_size",
         ]:
             if hasattr(self, prop):
                 delattr(self, prop)
 
+    def _reactivate_effect(self):
+        self.clear_transition_effect()
+        self.transitions = Transitions(self.effective_pixel_count)
+        if self._active_effect is not None:
+            self._active_effect._deactivate()
+            if self.pixel_count > 0:
+                self._active_effect.activate(self)
+
     def update_segments(self, segments_config):
-        self.lock.acquire()
-        segments_config = [list(item) for item in segments_config]
-        _segments = self.SEGMENTS_SCHEMA(segments_config)
+        """
+        Update the segments of the virtual with the given configuration.
 
-        _pixel_count = self.pixel_count
+        Args:
+            segments_config (list): A list of segment configurations.
 
-        if _segments != self._segments:
-            if self._active:
-                self.deactivate_segments()
-                # try to register this new set of segments
-                # if it fails, restore previous segments and raise the error
-                try:
-                    self.activate_segments(_segments)
-                except ValueError:
+        Raises:
+            ValueError: If the new set of segments cannot be activated.
+
+        Returns:
+            None
+        """
+        with self.lock:
+            segments_config = [list(item) for item in segments_config]
+            _segments = self.SEGMENTS_SCHEMA(segments_config)
+
+            _pixel_count = self.pixel_count
+
+            if _segments != self._segments:
+                if self._active:
                     self.deactivate_segments()
-                    self.activate_segments(self._segments)
-                    raise
+                    # try to register this new set of segments
+                    # if it fails, restore previous segments and raise the error
+                    try:
+                        self.activate_segments(_segments)
+                    except ValueError:
+                        self.deactivate_segments()
+                        self.activate_segments(self._segments)
+                        raise
 
-            self._segments = _segments
+                self._segments = _segments
 
-            self.invalidate_cached_props()
+                self.invalidate_cached_props()
 
+                # Restart active effect if total pixel count has changed
+                # eg. devices might be reordered, but total pixel count is same
+                # so no need to restart the effect
+                if self.pixel_count != _pixel_count:
+                    # chenging segments is a deep edit, just flush any transition
+                    self._reactivate_effect()
+
+                mode = self._config["transition_mode"]
+                self.frame_transitions = self.transitions[mode]
+            # Update internal config with new segment
+            for idx, item in enumerate(self._ledfx.config["virtuals"]):
+                if item["id"] == self.id:
+                    item["segments"] = self._segments
+                    self._ledfx.config["virtuals"][idx] = item
+                    break
             _LOGGER.debug(
                 f"Virtual {self.id}: updated with {len(self._segments)} segments, totalling {self.pixel_count} pixels"
             )
 
-            # Restart active effect if total pixel count has changed
-            # eg. devices might be reordered, but total pixel count is same
-            # so no need to restart the effect
-            if self.pixel_count != _pixel_count:
-                # chenging segments is a deep edit, just flush any transition
-                self.clear_transition_effect()
-                self.transitions = Transitions(self.pixel_count)
-                if self._active_effect is not None:
-                    self._active_effect._deactivate()
-                    if self.pixel_count > 0:
-                        self._active_effect.activate(self)
-
-            mode = self._config["transition_mode"]
-            self.frame_transitions = self.transitions[mode]
-
-        self.lock.release()
-
     def set_preset(self, preset_info):
+        """
+        Sets the preset for the virtual.
+
+        Args:
+            preset_info (tuple): A tuple containing the category, effect_id, and preset_id of the preset.
+
+        Returns:
+            None
+        """
         category, effect_id, preset_id = preset_info
 
         # Create the effect and add it to the virtual
@@ -291,42 +340,56 @@ class Virtual:
         self.set_effect(effect)
 
     def set_effect(self, effect):
-        self.lock.acquire()
-        if not self._devices:
-            error = f"Virtual {self.id}: Cannot activate, no configured device segments"
-            _LOGGER.warning(error)
-            raise ValueError(error)
+        """
+        Sets the active effect for the virtual device.
 
-        if (
-            self._config["transition_mode"] != "None"
-            and self._config["transition_time"] > 0
-        ):
-            self.transition_frame_total = (
-                self.refresh_rate * self._config["transition_time"]
-            )
-            self.transition_frame_counter = 0
-            self.clear_transition_effect()
+        Args:
+            effect: The effect to set as the active effect.
 
-            if self._active_effect is None:
-                self._transition_effect = DummyEffect(self.pixel_count)
+        Raises:
+            ValueError: If no configured device segments are available.
+            RuntimeError: If an error occurs while setting the active effect.
+
+        """
+        with self.lock:
+            if not self._devices:
+                error = f"Virtual {self.id}: Cannot activate, no configured device segments"
+                _LOGGER.warning(error)
+                raise ValueError(error)
+
+            if (
+                self._config["transition_mode"] != "None"
+                and self._config["transition_time"] > 0
+            ):
+                self.transition_frame_total = (
+                    self.refresh_rate * self._config["transition_time"]
+                )
+                self.transition_frame_counter = 0
+                self.clear_transition_effect()
+
+                if self._active_effect is None:
+                    self._transition_effect = DummyEffect(
+                        self.effective_pixel_count
+                    )
+                else:
+                    self._transition_effect = self._active_effect
             else:
-                self._transition_effect = self._active_effect
-        else:
-            # no transition effect to clean up, so clear the active effect now!
-            self.clear_active_effect()
-            self.clear_transition_effect()
+                # no transition effect to clean up, so clear the active effect now!
+                self.clear_active_effect()
+                self.clear_transition_effect()
 
-        self._active_effect = effect
-        self._active_effect.activate(self)
-        self._ledfx.events.fire_event(
-            EffectSetEvent(
-                self._active_effect.name,
-                self._active_effect.id,
-                self.active_effect.config,
-                self.id,
+            self.flush_pending_clear_frame()
+
+            self._active_effect = effect
+            self._active_effect.activate(self)
+            self._ledfx.events.fire_event(
+                EffectSetEvent(
+                    self._active_effect.name,
+                    self._active_effect.id,
+                    self.active_effect.config,
+                    self.id,
+                )
             )
-        )
-        self.lock.release()
         try:
             self.active = True
         except RuntimeError:
@@ -342,29 +405,35 @@ class Virtual:
         self._active_effect = None
 
     def clear_effect(self):
-        self.lock.acquire()
-        self._ledfx.events.fire_event(EffectClearedEvent())
-        self.clear_transition_effect()
+        with self.lock:
+            self._ledfx.events.fire_event(EffectClearedEvent())
+            self.clear_transition_effect()
 
-        if (
-            self._config["transition_mode"] != "None"
-            and self._config["transition_time"] > 0
-        ):
-            self._transition_effect = self._active_effect
-            self._active_effect = DummyEffect(self.pixel_count)
+            if (
+                self._config["transition_mode"] != "None"
+                and self._config["transition_time"] > 0
+            ):
+                self._transition_effect = self._active_effect
+                self._active_effect = DummyEffect(self.effective_pixel_count)
 
-            self.transition_frame_total = (
-                self.refresh_rate * self._config["transition_time"]
+                self.transition_frame_total = (
+                    self.refresh_rate * self._config["transition_time"]
+                )
+                self.transition_frame_counter = 0
+            else:
+                # no transition effect to clean up, so clear the active effect now!
+                self.clear_active_effect()
+
+            self.flush_pending_clear_frame()
+
+            self.clear_handle = self._ledfx.loop.call_later(
+                self._config["transition_time"], self.clear_frame
             )
-            self.transition_frame_counter = 0
-        else:
-            # no transition effect to clean up, so clear the active effect now!
-            self.clear_active_effect()
 
-        self._ledfx.loop.call_later(
-            self._config["transition_time"], self.clear_frame
-        )
-        self.lock.release()
+    def flush_pending_clear_frame(self):
+        if self.clear_handle is not None:
+            self.clear_handle.cancel()
+            self.clear_handle = None
 
     def clear_transition_effect(self):
         if self._transition_effect is not None:
@@ -377,31 +446,57 @@ class Virtual:
         self._active_effect = None
 
     def clear_frame(self):
-        self.lock.acquire()
-        self.clear_active_effect()
-        self.clear_transition_effect()
+        """
+        Clears the frame by performing the following steps:
+        1. Clears the active effect.
+        2. Clears the transition effect.
+        3. If the virtual device is active:
+           - Clears all the pixel data by setting it to zeros.
+           - Flushes the assembled frame to the device.
+           - Fires a VirtualUpdateEvent to notify listeners of the updated frame.
+           - Releases the lock.
+           - Deactivates the virtual device.
+        """
+        # Little tricky logic here - we need to clear the active effect and
+        # transition effect before we flush the frame, but we need to flush
+        # the frame before we deactivate the virtual device. We also need to
+        # make sure that we don't clear the frame if the virtual device is
+        # not active.
+        # All of this requires thread lock management that's a bit unwieldy
 
-        if self._active:
-            # Clear all the pixel data before deactivating the device
-            self.assembled_frame = np.zeros((self.pixel_count, 3))
-            self.flush(self.assembled_frame)
-            self._ledfx.events.fire_event(
-                VirtualUpdateEvent(self.id, self.assembled_frame)
-            )
-            self.lock.release()
+        assembled_frame = None
+        with self.lock:
+            self.clear_active_effect()
+            self.clear_transition_effect()
+            if self._active:
+                assembled_frame = np.zeros((self.pixel_count, 3))
+                self.flush(assembled_frame)
+                self._fire_update_event(assembled_frame)
+
+        # Deactivate the device - this requires the thread lock
+        # Hence why we do it outside of the lock and after the frame is cleared
+        # This is because the deactivate method will join the thread
+        # and we don't want to call join while holding the lock
+        if assembled_frame is not None:
             self.deactivate()
-        else:
-            self.lock.release()
 
     def force_frame(self, color):
         """
         Force all pixels in device to color
         Use for pre-clearing in calibration scenarios
         """
-        self.assembled_frame = np.full((self.pixel_count, 3), color)
+        self.assembled_frame = np.full((self.effective_pixel_count, 3), color)
         self.flush(self.assembled_frame)
+        self._fire_update_event()
+
+    def _fire_update_event(self, frame=None):
+        if frame is None:
+            frame = self.assembled_frame
+
         self._ledfx.events.fire_event(
-            VirtualUpdateEvent(self.id, self.assembled_frame)
+            VirtualUpdateEvent(
+                self.id, self._effective_to_physical_pixels(frame)
+            )
         )
 
     def oneshot(self, color, ramp, hold, fade):
@@ -488,28 +583,25 @@ class Virtual:
             start_time = timeit.default_timer()
             # we need to lock before we test, or we could deactivate
             # between test and execution
-            self.lock.acquire()
-            if (
-                self._active_effect
-                and self._active_effect.is_active
-                and hasattr(self._active_effect, "pixels")
-            ):
-                # self.assembled_frame = await self._ledfx.loop.run_in_executor(
-                #     self._ledfx.thread_executor, self.assemble_frame
-                # )
-                self.assembled_frame = self.assemble_frame()
-                if self.assembled_frame is not None and not self._paused:
-                    if not self._config["preview_only"]:
-                        # self._ledfx.thread_executor.submit(self.flush)
-                        # await self._ledfx.loop.run_in_executor(
-                        #     self._ledfx.thread_executor, self.flush
-                        # )
-                        self.flush()
+            with self.lock:
+                if (
+                    self._active_effect
+                    and self._active_effect.is_active
+                    and hasattr(self._active_effect, "pixels")
+                ):
+                    # self.assembled_frame = await self._ledfx.loop.run_in_executor(
+                    #     self._ledfx.thread_executor, self.assemble_frame
+                    # )
+                    self.assembled_frame = self.assemble_frame()
+                    if self.assembled_frame is not None and not self._paused:
+                        if not self._config["preview_only"]:
+                            # self._ledfx.thread_executor.submit(self.flush)
+                            # await self._ledfx.loop.run_in_executor(
+                            #     self._ledfx.thread_executor, self.flush
+                            # )
+                            self.flush()
 
-                    self._ledfx.events.fire_event(
-                        VirtualUpdateEvent(self.id, self.assembled_frame)
-                    )
-            self.lock.release()
+                        self._fire_update_event()
 
             # adjust for the frame assemble time, min allowed sleep 1 ms
             # this will be more frame accurate on high res sleep systems
@@ -665,6 +757,10 @@ class Virtual:
         if self._os_active:
             self.oneshot_weight()
 
+        if self._config["mapping"] == "span":
+            # In span mode we can calculate the final pixels once for all segments
+            pixels = self._effective_to_physical_pixels(pixels)
+
         color_cycle = itertools.cycle(color_list)
 
         for device_id, segments in self._segments_by_device.items():
@@ -696,10 +792,21 @@ class Virtual:
                             device_start,
                             device_end,
                         ) in segments:
-                            target_len = device_end - device_start + 1
-                            seg = interpolate_pixels(pixels, target_len)[
-                                ::step
-                            ]
+                            target_physical_len = device_end - device_start + 1
+                            target_effect_len = (
+                                self._get_effective_pixel_count(
+                                    target_physical_len
+                                )
+                            )
+                            # In copy mode, we need to scale the effect and afterwards expand the
+                            # pixel groups separately for every segment, because pre-calculating once
+                            # and scaling would lead to incorrect pixel group lengths.
+                            seg = interpolate_pixels(
+                                pixels, target_effect_len
+                            )[::step]
+                            seg = self._effective_to_physical_pixels(
+                                seg, target_physical_len
+                            )
                             if self._os_active:
                                 self.oneshot_apply(seg)
                             data.append((seg, device_start, device_end))
@@ -851,6 +958,7 @@ class Virtual:
             _config = new_config
 
         _config = self.CONFIG_SCHEMA(_config)
+        reactivate_effect = False
 
         if hasattr(self, "_config"):
             if _config["mapping"] != self._config["mapping"]:
@@ -918,6 +1026,11 @@ class Virtual:
                         if hasattr(self._active_effect, "set_init"):
                             self._active_effect.set_init()
 
+                    if _config["grouping"] != self._config["grouping"]:
+                        # The effect needs to be reactivated later after the config has been applied
+                        reactivate_effect = True
+                        self.invalidate_cached_props()
+
         setattr(self, "_config", _config)
 
         self.frequency_range = FrequencyRange(
@@ -928,6 +1041,48 @@ class Virtual:
             VirtualConfigUpdateEvent(self.id, self._config)
         )
 
+        if reactivate_effect:
+            self._reactivate_effect()
+
+    @cached_property
+    def effective_pixel_count(self):
+        """The number of pixels to calculate by effects.
+
+        Can be less than the number of physical pixels (:attr:`~pixel_count`) when
+        pixel grouping (:attr:`~group_size`) is activated.
+        """
+        return self._get_effective_pixel_count(self.pixel_count)
+
+    @cached_property
+    def group_size(self):
+        """The number of physical pixels to group into virtual effect pixels."""
+        grouping = self._config["grouping"]
+
+        if grouping is None or grouping < 1:
+            return 1
+
+        return grouping
+
+    def _get_effective_pixel_count(self, physical_pixel_count):
+        """Calculates the number of effective pixels for a given number of physical pixels, considering pixel grouping."""
+        return int(np.ceil(physical_pixel_count / self.group_size))
+
+    def _effective_to_physical_pixels(
+        self, effective_pixels, pixel_count=None
+    ):
+        """Projects an array of effective pixels into an array of pixels for physical rendering, considering pixel grouping."""
+        if self.group_size <= 1:
+            return effective_pixels
+
+        if not pixel_count:
+            pixel_count = self.pixel_count
+
+        effective_pixels = np.repeat(
+            effective_pixels, self.group_size, axis=0
+        )[:pixel_count, :]
+
+        return effective_pixels
+
 
 class Virtuals:
     """Thin wrapper around the device registry that manages virtuals"""
@@ -1043,3 +1198,35 @@ class Virtuals:
 
     def get(self, *args):
         return self._virtuals.get(*args)
+
+
+def update_effect_config(config, virtual_id, effect):
+    """
+    Update the configuration of a virtual effect.
+
+    This function is important for maintaining both the active effect and adding
+    to the collection of effects configs on the virtual
+
+    Args:
+        config (dict): The current config structure
+        virtual_id (str): The ID of the virtual effect.
+        effect (Effect): The effect object containing the updated configuration.
+
+    Returns:
+        None
+    """
+    # Store as both the active effect to protect existing code, and one of effects
+    virtual = next(
+        (item for item in config["virtuals"] if item["id"] == virtual_id),
+        None,
+    )
+    if virtual:
+        if not ("effects" in virtual):
+            virtual["effects"] = {}
+        virtual["effects"][effect.type] = {}
+        virtual["effects"][effect.type]["type"] = effect.type
+        virtual["effects"][effect.type]["config"] = effect.config
+        if not ("effect" in virtual):
+            virtual["effect"] = {}
+        virtual["effect"]["type"] = effect.type
+        virtual["effect"]["config"] = effect.config
